open Base
open Oksql

include Application_intf
include Application_intf.Types

type t = application [@@deriving equal]

module App_from_sql = struct
  type t = application

  let of_sql_row row =
    let open Sqlite3.Data in
    let open Crypto in
    match[@warning "-4"] row with
    | [| "public_key", TEXT pk;
         "name", TEXT name;
         "signing_key", TEXT sk;
      |] ->
      let public_key = Public_key.of_base64_exn pk in
      let signing_key = Public_key.of_base64_exn sk in
      { public_key; name; signing_key }
    | _ ->
      failwith "Application.of_sql_row: invalid data"
end

let q_find_all = {|
SELECT public_key, name, signing_key
FROM applications;
|}

let q_insert_app = {|
INSERT INTO applications
(public_key, name, signing_key)
VALUES
(:public_key, :name, :signing_key);
|}

let q_remove_app = {|
DELETE FROM applications
WHERE public_key = :public_key;
|}

let q_find_by_public_key = {|
SELECT public_key, name, signing_key
FROM applications
WHERE public_key = :public_key;
|}

let find_all db =
  exec_fetch (module App_from_sql) db q_find_all

let create db ~public_key ~name ~signing_key =
  let open Sqlite3.Data in
  let open Crypto in
  exec_only db q_insert_app ~params:[
    ":public_key", TEXT (Public_key.to_base64 public_key);
    ":name", TEXT name;
    ":signing_key", TEXT (Public_key.to_base64 signing_key);
  ];
  { public_key; name; signing_key; }

let remove db ~public_key =
  exec_only db q_remove_app ~params:[
    ":public_key", Sqlite3.Data.TEXT (Crypto.Public_key.to_base64 public_key)
  ]

let find_by_public_key db ~public_key =
  let results = exec_fetch (module App_from_sql) db q_find_by_public_key ~params:[
      ":public_key", Sqlite3.Data.TEXT (Crypto.Public_key.to_base64 public_key)
    ] in
  match results with
  | [| app |] -> Some app
  | [| |] -> None
  | _ -> failwith "Application.find_by_public_key: more than one application is found"
