module Types = struct
  type application = {
    public_key: Crypto.Public_key.t;
    name: String.t;
    signing_key: Crypto.Public_key.t;
  } [@@deriving equal]
end

module type Application = sig
  include module type of Types
  type t = application [@@deriving equal]

  (** Creates new application entry in the database *)
  val create:
    Sqlite3.db ->
    public_key: Crypto.Public_key.t ->
    name: string ->
    signing_key: Crypto.Public_key.t ->
    t

  (** Removes application from the database *)
  val remove:
    Sqlite3.db ->
    public_key: Crypto.Public_key.t ->
    unit

  (** Retrieves all applications from the database *)
  val find_all:
    Sqlite3.db ->
    t array

  (** Retrieve one application by its public key *)
  val find_by_public_key:
    Sqlite3.db ->
    public_key: Crypto.Public_key.t ->
    t option
end
