open Alcotest
open Base
open Oksql
open Okauth
open Okauth.Crypto
open Okauth.Application
open Utils

let test_should_create_app =
  test_case "should insert app into the database" `Quick begin fun () ->
    with_okauth_db_in_mem begin fun db ->
      let (pek, _) = generate_keypair ~purpose:Encryption in
      let (psk, _) = generate_keypair ~purpose:Signing in
      let name = "Test App Name" in
      let returned_app = Application.create db ~public_key:pek ~name:name ~signing_key:psk in
      let app_in_db = Option.value_exn (Application.find_by_public_key db ~public_key:pek) in

      (check bool) "inserted app doesn't equal the returned one" true
        (Application.equal returned_app app_in_db);
      (check string) "names don't match" name app_in_db.name;
      (check bool) "public keys don't match" true
        (Public_key.equal pek app_in_db.public_key);
      (check bool) "signing keys don't match" true
        (Public_key.equal psk app_in_db.signing_key)
    end
  end

let test_should_remove_app =
  test_case "should remove app from the database" `Quick begin fun () ->
    with_okauth_db_in_mem begin fun db ->
      exec_only db "INSERT INTO applications (public_key, name, signing_key) VALUES ('dGVzdF9waw', 'Test App Name', 'dGVzdF9zaw')";
      let apps_before = Application.find_all db in
      (check int) "test app has not been inserted" 1 (Array.length apps_before);

      Application.remove db ~public_key:(Public_key.of_string "test_pk");
      let apps_after = Application.find_all db in
      (check int) "test app has not been removed" 0 (Array.length apps_after)
    end
  end

let cases = [
  test_should_create_app;
  test_should_remove_app;
]
